/**
 * section: Tree
 * synopsis: Navigates a tree to print element names
 * purpose: Parse a file to a tree, use xmlDocGetRootElement() to
 *          get the root element, then walk the document and print
 *          all the element name in document order.
 * usage: tree1 filename_or_URL
 * test: tree1 test2.xml > tree1.tmp ; diff tree1.tmp tree1.res ; rm tree1.tmp
 * author: Dodji Seketeli
 * copy: see Copyright for the status of this software.
 */
#include <stdio.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/HTMLparser.h>
#include <libxml/tree.h>

#ifdef LIBXML_TREE_ENABLED

xmlChar * trim(const xmlChar *src , xmlChar *dest) {
	int firstNonWS=0;
	int lastNonWS=0;
	int i;

	for(i=0; 
            src[i] 
            && (src[i]==' ' || src[i]=='\n' || src[i]=='\t'|| src[i]=='\r');
            i++) {}
	firstNonWS=i;

	for(i=strlen((const char *)src)-1; 
            i>=0 && 
            (src[i]==' ' || src[i]=='\n' || src[i]=='\t'|| src[i]=='\r');
            i--) {}
	lastNonWS=i;

//	printf("trim f %d c %d l %d c %d d %d\n", firstNonWS, src[firstNonWS], lastNonWS, src[lastNonWS], lastNonWS-firstNonWS+1);


	for(i=firstNonWS; i<=lastNonWS; i++) {
		dest[i-firstNonWS] = src[i];
        }

	if(lastNonWS+1-firstNonWS>=0)
	   dest[lastNonWS+1-firstNonWS]='\0';
	else 
	   dest[0]='\0';

	return dest;
}



/*
 *To compile this file using gcc you can type
 *gcc `xml2-config --cflags --libs` -o xmlexample libxml2-example.c
 */

/**
 * print_element_names:
 * @a_node: the initial xml node to consider.
 *
 * Prints the names of the all the xml elements
 * that are siblings or children of a given xml node.
 */
static void
print_element_names(xmlNode * a_node, int ind)
{
    xmlNode *cur_node = NULL;
    xmlChar aux[1024*16];
    static xmlChar *href = NULL;
    int child_ind;

//    if(a_node!=0) 
//        printf("print_element_names root_type %d\n", a_node->type);
    for (cur_node = a_node, child_ind=0; cur_node; cur_node = cur_node->next, child_ind++) {
        if (cur_node->type == XML_ELEMENT_NODE) {
	    if(xmlStrcmp(cur_node->name,(const xmlChar *)"a")==0) {
		href = xmlGetProp(cur_node,(const xmlChar *)"href"); //TODO: check memory leak
		if(href) {
                   printf("%*.*s%2d:%2d Element: n %s href %s\n", ind,ind, "", ind/2, child_ind,cur_node->name, href);
                }
		else {
                   printf("%*.*s%2d:%2d Element: n %s nohref\n", ind,ind, "", ind/2, child_ind,cur_node->name);
                }
            }
	    else {
                 printf("%*.*s%2d:%2d Element: n %s\n", ind,ind, "", ind/2, child_ind,cur_node->name);
            }
        }
        else if (cur_node->type == XML_TEXT_NODE) {
            printf("%*.*s%2d:%2d Text: n %s t \"%s\"\n", ind,ind, "", ind/2, child_ind,cur_node->name, trim(cur_node->content, aux));
        }

        print_element_names(cur_node->children, ind+2);
    }
}


/**
 * Simple example to parse a file called "file.xml", 
 * walk down the DOM, and print the name of the 
 * xml elements nodes.
 */
int
main(int argc, char **argv)
{
    xmlDoc *doc = NULL;
    xmlNode *root_element = NULL;

    if (argc != 2)
        return(1);

    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION

    /*parse the file and get the DOM */
    doc = xmlReadFile(argv[1], NULL, 0);

    if (doc == NULL) {
        printf("error: could not parse file %s\n", argv[1]);
    }

    /*Get the root element node */
    root_element = xmlDocGetRootElement(doc);

    print_element_names(root_element,0);

    /*free the document */
    xmlFreeDoc(doc);

    /*
     *Free the global variables that may
     *have been allocated by the parser.
     */
    xmlCleanupParser();

    return 0;
}
#else
int main(void) {
    fprintf(stderr, "Tree support not compiled in\n");
    exit(1);
}
#endif
